@@ -13,6 +13,7 @@ from k3color import green
 from k3handy import cmdpass
 from k3handy import pjoin
 from k3handy import to_bytes
+from k3fs import fread
 
 from .. import mistune
 
@@ -65,11 +66,6 @@ def code_join(n):
     return txt
 
 
-def code_to_html(n, ctx=None):
-    txt = code_join(n)
-    return k3down2.convert('code', txt, 'html').split('\n')
-
-
 def block_code_to_jpg(mdrender, n, width=None, ctx=None):
     txt = code_join(n)
 
@@ -166,6 +162,10 @@ def zhihu_specific(mdrender, n, ctx=None):
     return render_with_features(mdrender, n, ctx=ctx, features=zhihu_features)
 
 
+def minimal_mistake_specific(mdrender, n, ctx=None):
+    return render_with_features(mdrender, n, ctx=ctx, features=minimal_mistake_features)
+
+
 def wechat_specific(mdrender, n, ctx=None):
     return render_with_features(mdrender, n, ctx=ctx, features=wechat_features)
 
@@ -233,6 +233,7 @@ class MDRender(object):
         'zhihu': zhihu_specific,
         'wechat': wechat_specific,
         'weibo': weibo_specific,
+        'minimal_mistake': minimal_mistake_specific,
         'simple': simple_specific,
     }
 
@@ -454,7 +455,7 @@ def join_math_block(nodes):
     join_math_text(nodes)
 
 
-def parse_math(nodes):
+def parse_math(conf, nodes):
     """
     Extract all math segment such as ``$$ ... $$`` from a text and build a
     math_block or math_inline node.
@@ -465,10 +466,10 @@ def parse_math(nodes):
     for n in nodes:
 
         if 'children' in n:
-            n['children'] = parse_math(n['children'])
+            n['children'] = parse_math(conf, n['children'])
 
         if n['type'] == 'text':
-            new_children = extract_math(n)
+            new_children = extract_math(conf, n)
             children.extend(new_children)
         else:
             children.append(n)
@@ -501,22 +502,24 @@ def join_math_text(nodes):
             i += 1
 
 
-inline_math = r'\$\$(.*?)\$\$'
-
-
-def extract_math(n):
+def extract_math(conf, n):
     """
-    Extract ``$$ ... $$`` from a text node and build a new node.
+    Extract ``$$ ... $$`` or ``$ .. $` from a text node and build a new node.
     The original text node is split into multiple segments.
+
+    The math is a block if it is a paragraph.
+    Otherwise, it is an inline math.
     """
     children = []
 
+    math_regex = r'([$]|[$][$])([^$].*?)\1'
+
     t = n['text']
     while True:
-        match = re.search(inline_math, t, flags=re.DOTALL)
+        match = re.search(math_regex, t, flags=re.DOTALL)
         if match:
             children.append({'type': 'text', 'text': t[:match.start()]})
-            children.append({'type': 'math_inline', 'text': match.groups()[0]})
+            children.append({'type': 'math_inline', 'text': match.groups()[1]})
             t = t[match.end():]
 
             left = children[-2]['text']
@@ -592,21 +595,43 @@ def build_refs(meta):
 
 
 def replace_ref_with_def(nodes, refs):
+    """
+    Convert ``[text][link-def]`` to ``[text](link-url)``
+    Convert ``[link-def][]``     to ``[link-def](link-url)``
+    Convert ``[link-def]``       to ``[link-def](link-url)``
+    """
+
+    used_defs={}
+
     for n in nodes:
 
         if 'children' in n:
-            replace_ref_with_def(n['children'], refs)
+            used = replace_ref_with_def(n['children'], refs)
+            used_defs.update(used)
 
         if n['type'] == 'text':
             t = n['text']
-            link = re.match(r'\[(.*?)\](\[\])?', t)
-            if link:
-                txt = link.groups()[0]
-                if txt in refs:
-                    n['type'] = 'link'
-                    r = refs[txt]
-                    n['link'] = r.split()[0]
-                    n['children'] = [{'type': 'text', 'text': txt}]
+            link = re.match(r'^\[(.*?)\](\[([0-9a-zA-Z_\-]*?)\])?$', t)
+            if not link:
+                continue
+
+            gs = link.groups()
+            txt = gs[0]
+            if len(gs) >= 3:
+                definition = gs[2]
+
+            if definition is None or definition == '':
+                definition = txt
+
+            if definition in refs:
+                n['type'] = 'link'
+                r = refs[definition]
+                #  TODO title
+                n['link'] = r.split()[0]
+                n['children'] = [{'type': 'text', 'text': txt}]
+                used_defs[definition] = r
+
+    return used_defs
 
 
 def new_parser():
@@ -651,11 +676,19 @@ def render_ref_list(refs, platform):
     ref_lines = ["", "Reference:", ""]
     for _id, d in refs.items():
         #  d is in form "<url> <alt>"
-        url = d.split()[0]
+        url_alt = d.split()
+        url = url_alt[0]
+
+        if len(url_alt) == 1:
+            txt = _id
+        else:
+            txt = ' '.join(url_alt[1:])
+            txt = txt.strip('"')
+            txt = txt.strip("'")
 
         ref_lines.append(
             '- {id} : [{url}]({url})'.format(
-                id=_id, url=url
+                id=txt, url=url
             )
         )
 
@@ -701,7 +734,6 @@ class AssetRepo(object):
         gu = k3git.GitUrl.parse(repo_url)
         f = gu.fields
 
-        url = ""
         if (f['scheme'] == 'https'
                 and 'committer' in f
                 and 'token' in f):
@@ -855,6 +887,15 @@ zhihu_features = dict(
     )
 )
 
+#  jekyll theme: minimal mistake
+minimal_mistake_features = dict(
+    image=image_local_to_remote,
+    block_code=dict(
+        mermaid=block_code_mermaid_to_jpg,
+        graphviz=block_code_graphviz_to_jpg,
+    )
+)
+
 
 # type, subtype... action
 #
@@ -941,11 +982,14 @@ class Config(object):
                  src_path,
                  platform,
                  output_dir,
-                 asset_output_dir, 
+                 asset_output_dir,
                  asset_repo_url=None,
                  md_output_path=None,
                  code_width=1000,
                  keep_meta=None,
+                 ref_files=None,
+                 jekyll=False,
+                 rewrite=None,
                  ):
         """
         Config of markdown rendering
@@ -972,22 +1016,37 @@ class Config(object):
         self.platform = platform
         self.src_path = src_path
 
+
         self.code_width = code_width
         if keep_meta is None:
             keep_meta = False
         self.keep_meta = keep_meta
 
+        if ref_files is None:
+            ref_files = []
+        self.ref_files = ref_files
+
+        self.jekyll = jekyll
+
+        if rewrite is None:
+            rewrite = []
+        self.rewrite = rewrite
+
         fn = os.path.split(self.src_path)[-1]
 
-        # jekyll style
-        fnm = re.match(r'\d\d\d\d-\d\d-\d\d-(.*)', fn)
-        if fnm:
-            fn = fnm.groups()[0]
+        trim_fn = re.match(r'\d\d\d\d-\d\d-\d\d-(.*)', fn)
+        if trim_fn:
+            trim_fn = trim_fn.groups()[0]
+        else:
+            trim_fn = fn
 
-        self.article_name = fn.rsplit('.', 1)[0]
+        if not self.jekyll:
+            fn = trim_fn
 
-        self.rel_dir = self.article_name
-        self.asset_output_dir = pjoin(asset_output_dir, self.rel_dir)
+        self.article_name = trim_fn.rsplit('.', 1)[0]
+
+        self.asset_output_dir = pjoin(asset_output_dir, self.article_name)
+        self.rel_dir = os.path.relpath(self.asset_output_dir, self.output_dir)
 
         assert(self.md_output_path is not None)
 
@@ -1014,9 +1073,14 @@ class Config(object):
             msg(darkyellow(k), ": ",  getattr(self, k))
 
     def img_url(self, fn):
-        return self.asset_repo.path_pattern.format(
+        url = self.asset_repo.path_pattern.format(
             path=pjoin(self.rel_dir, fn))
 
+        for (pattern, repl) in self.rewrite:
+            url = re.sub(pattern, repl, url)
+
+        return url
+
     def push(self):
         x = dict(cwd=self.output_dir)
 
@@ -1033,7 +1097,7 @@ class Config(object):
                 **x)
         cmdpass('git', 'push', '-f', self.asset_repo.url,
                 'HEAD:refs/heads/' + self.asset_repo.branch, **x)
-        
+
         if not has_git:
             msg("Removing tmp git dir: ", self.output_dir + '/.git')
             shutil.rmtree(self.output_dir + '/.git')
@@ -1051,7 +1115,20 @@ def convert_md(conf, handler=None):
     cont, meta, meta_text = extract_jekyll_meta(cont)
     cont, article_refs = extract_ref_definitions(cont)
 
-    refs = build_refs(meta)
+    refs = {}
+
+    for ref_path in conf.ref_files:
+        fcont = fread(ref_path)
+        y = yaml.safe_load(fcont)
+        for r in y.get('universal', []):
+            refs.update(r)
+        for r in y.get(conf.platform, []):
+            refs.update(r)
+
+
+    meta_refs = build_refs(meta)
+    refs.update(meta_refs)
+
     refs.update(article_refs)
 
     parse_to_ast = new_parser()
@@ -1060,22 +1137,27 @@ def convert_md(conf, handler=None):
     #  with open('ast', 'w') as f:
     #      f.write(pprint.pformat(ast))
 
-    fix_tables(ast)
+    # TODO use feature detection to decide if we need to convert table to hml
+    if conf.platform == 'minimal_mistake':
+        #  jekyll output does render table well.
+        pass
+    else:
+        fix_tables(ast)
 
     #  with open('fixed-table', 'w') as f:
     #      f.write(pprint.pformat(ast))
 
-    replace_ref_with_def(ast, refs)
+    used_refs = replace_ref_with_def(ast, refs)
 
     # extract already inlined math
-    ast = parse_math(ast)
+    ast = parse_math(conf, ast)
 
     #  with open('after-math-1', 'w') as f:
     #      f.write(pprint.pformat(ast))
 
     # join cross paragraph math
     join_math_block(ast)
-    ast = parse_math(ast)
+    ast = parse_math(conf, ast)
 
     #  with open('after-math-2', 'w') as f:
     #  f.write(pprint.pformat(ast))
@@ -1092,7 +1174,7 @@ def convert_md(conf, handler=None):
 
     out.append('')
 
-    ref_list = render_ref_list(refs, conf.platform)
+    ref_list = render_ref_list(used_refs, conf.platform)
     out.extend(ref_list)
 
     out.append('')
@@ -1100,7 +1182,7 @@ def convert_md(conf, handler=None):
     ref_lines = [
         '[{id}]: {d}'.format(
             id=_id, d=d
-        ) for _id, d in refs.items()
+        ) for _id, d in used_refs.items()
     ]
     out.extend(ref_lines)
 
@@ -1156,9 +1238,9 @@ def main():
     parser.add_argument('-p', '--platform', action='store',
                         required=False,
                         default='zhihu',
-                        choices=["zhihu", "wechat", "weibo", "simple"],
+                        choices=["zhihu", "wechat", "weibo", "simple", "minimal_mistake"],
                         help='convert to a platform compatible format.'
-                        'simple is a special type that it produce simplest output, only plain text and images, there wont be table, code block, math etc.'
+                        ' simple is a special type that it produce simplest output, only plain text and images, there wont be table, code block, math etc.'
                         )
 
     parser.add_argument('--keep-meta', action='store_true',
@@ -1168,6 +1250,36 @@ def main():
                         ' default: False'
                         )
 
+    parser.add_argument('--jekyll', action='store_true',
+                        required=False,
+                        default=False,
+                        help='respect jekyll syntax: 1) implies <keep-meta>: do not trim md header meta;'
+                        ' 2) keep jekyll style name: YYYY-MM-DD-TITLE.md;'
+                        ' default: False'
+                        )
+
+    parser.add_argument('--refs', action='append',
+                        required=False,
+                        help='external file that contains ref definitions'
+                        ' A ref file is a yaml contains dict of list.'
+                        ' A dict key is the platform name, only visible to <platform> argument'
+                        ' "univeral" is visible with any <platform>'
+                        ' An example of ref file data:'
+                        '  {"universal": [{"grpc":"http:.."}, {"protobuf":"http:.."}],'
+                        '   "zhihu": [{"grpc":"http:.."}, {"protobuf":"http:.."}]'
+                        '}.'
+                        ' default: []'
+                        )
+
+    parser.add_argument('--rewrite', action='append',
+                        nargs=2,
+                        required=False,
+                        help='rewrite generated to image url.'
+                        ' E.g.: --rewrite "/asset/" "/resource/"'
+                        ' will transform "/asset/banner.jpg" to "/resource/banner.jpg"'
+                        ' default: []'
+                        )
+
     parser.add_argument('--code-width', action='store',
                         required=False,
                         default=1000,
@@ -1183,6 +1295,9 @@ def main():
     if args.asset_output_dir is None:
         args.asset_output_dir = args.output_dir
 
+    if args.jekyll:
+        args.keep_meta = True
+
     msg("Build markdown: ", darkyellow(args.src_path),
         " into ", darkyellow(args.md_output))
     msg("Build assets to: ", darkyellow(args.asset_output_dir))
@@ -1191,6 +1306,7 @@ def main():
 
     for path in args.src_path:
 
+        #  TODO Config should accept only two arguments: the path and a args
         conf = Config(
             path,
             args.platform,
@@ -1200,6 +1316,9 @@ def main():
             md_output_path=args.md_output,
             code_width=args.code_width,
             keep_meta=args.keep_meta,
+            ref_files=args.refs,
+            jekyll=args.jekyll,
+            rewrite=args.rewrite,
         )
 
         convert_md(conf)
